==> pycaptions-0.4.1-py3-none-any.whl: pycaptions/block.py <==
import budoux
import textwrap
from collections import defaultdict
from .microTime import MicroTime as MT
from .styling import Styling

class BlockType:
    CAPTION = 1
    COMMENT = 2
    STYLE = 3
    LAYOUT = 4
    METADATA = 5


class Block:
    """
    Represents a block of content in a multimedia file or document.

    Methods:
        getLines: Format text of specific language into multiple lines
        get: Get the text content of the block for a specific language.
        append: Append text to the block for a specific language.
        shift_time: Shift start and end times of the block by a specified duration.
        shift_start: Shift start time of the block by a specified duration.
        shif_end: Shift end time of the block by a specified duration.
        copy: Returns a copy of the current block.
        shift_time_us: Shift time of the block by microseconds.
        shift_start_us: Shift start time of the block by microseconds.
        shift_end_us: Shift end time of the block by microseconds.
        __getitem__: Retrieve the text of specific language.
        __setitem__: Set the text of specific language.
        __str__: Return a string representation of the block.
        __iadd__: In-place addition for the Block.
        __add__: Addition for the Blocks.
        __isub__: In-place subtraction for a specific language.
        __sub__: Subtraction for a specific language.
        __iter__: Iterator for iterating through the block languages.
        __next__: Iterator method returning a tuple of language and text.
    """
    def __init__(self, block_type: int, lang: str = "und", start_time: MT = None,
                 end_time: MT = None, text: str = "", **options):
        """
        Initialize a new instance of the Block class.

        Parameters:
        - block_type (int): The type of the block, represented as an integer, options in BlockType class
        - lang (str, optional): The language of the text in the block (default is "und" for undefined).
        - start_time (int, optional): The starting time of the block in microseconds (default is 0).
        - end_time (int, optional): The ending time of the block in microseconds (default is 0).
        - text (str, optional): The content of the block (default is an empty string).
        - **options: Additional keyword arguments for customization (e.g style, layout, ...).
        """
        self.block_type = block_type
        self.languages = defaultdict(str)
        if options.get("languages"):
            for i, j in options.get("languages").items():
                self.languages[i] = j
            del options["languages"]
        self.default_language = lang
        if text:
            self.languages[lang] = text.strip()
        self.start_time = start_time
        self.end_time = end_time
        self.options = options or {}

    def __getitem__(self, index: str):
        return self.languages[index]

    def __setitem__(self, index: str, value: str):
        self.languages[index] = value

    def __str__(self):
        temp = '\n'.join(f" {lang}: {text}" for lang, text in self.languages.items())
        return f"start: {self.start_time} end: {self.end_time}\n{temp}"

    def __iadd__(self, value):
        if not isinstance(value, Block):
            raise ValueError("Unsupported type. Must be an instance of `Block`")
        for key, language in value:
            self.languages[key] = language
        return self

    def __add__(self, value):
        if not isinstance(value, Block):
            raise ValueError("Unsupported type. Must be an instance of `Block`")
        out = Block(block_type=self.block_type, start_time=self.start_time,
                    end_time=self.end_time, lang=self.default_language, options=self.options)
        out.languages = self.languages.copy()
        for key, language, comment in value:
            out.languages[key] = language
        return out

    def __isub__(self, language: str):
        if language in self.languages:
            del self.languages[language]
        return self

    def __sub__(self, language: str):
        out = Block(block_type=self.block_type, start_time=self.start_time,
                    end_time=self.end_time, lang=self.default_language, options=self.options)
        out.languages = self.languages.copy()
        if language in out.languages:
            del out.languages[language]
        return out

    def __iter__(self):
        self._keys_iterator = iter(self.languages)
        return self

    def __next__(self):
        try:
            key = next(self._keys_iterator)
            return key, self.languages.get(key)
        except StopIteration:
            raise StopIteration

    def copy(self):
        return Block(self.block_type, self.default_language, self.start_time,
                     self.end_time, languages=self.languages, **self.options)

    def getLines(self, lang: str = "und", lines: int = 0) -> list[str]:
        """
        Format text of specific language into multiple lines.

        Args:
            lang (str, optional): Language code (default is "und" for undefined).
            lines (int, optional): The number of lines to format to. (default is 0 - autoformat).

        Returns:
            list[str]: A list of text lines.
        """
        text = self.get(lang)
        if lang == "ja":
            parser = budoux.load_default_japanese_parser()
            return parser.parse(text)
        elif lang in ["zh", "zh-CN", "zh-SG", "zh-Hans"]:
            parser = budoux.load_default_simplified_chinese_parser()
            return parser.parse(text)
        elif lang in ["zh-HK", "zh-MO", "zh-TW", "zh-Hant"]:
            parser = budoux.load_default_simplified_chinese_parser()
            return parser.parse(text)
        else:
            return textwrap.wrap(text)

    def get(self, lang: str) -> str:
        return Styling(self.languages.get(lang), "html.parser").get_text()
    
    def get_style(self, lang: str) -> str:
        return Styling(self.languages.get(lang), "html.parser")

    def append(self, text: str, lang: str = "und"):
        if not self.default_language:
            self.default_language = lang
        if lang not in ["ja", "zh", "zh-CN", "zh-SG", "zh-Hans",
                        "zh-HK", "zh-MO", "zh-TW", "zh-Hant"]:
            if self.languages[lang]:
                self.languages[lang] += " " + text.strip()
            else:
                self.languages[lang] = text.strip()
        else:
            self.languages[lang] += text

    def append_without_common_part(self, text: str, lang: str):
        common_lenght = 0
        current = self.get(lang)
        min_length = min(len(current), len(text))

        for i in range(min_length):
            if current[-i:] == text[:i]:
                common_lenght = i
        
        self.languages[lang] = current + text[common_lenght:]

    def shift_time_us(self, microseconds: int):
        self.start_time += microseconds
        self.end_time += microseconds

    def shift_time(self, time: MT):
        self.start_time += time
        self.end_time += time

    def shift_start_us(self, microseconds: int):
        self.start_time += microseconds

    def shift_start(self, time: MT):
        self.start_time += time

    def shift_end_us(self, microseconds: int):
        self.end_time += microseconds

    def shift_end(self, time: MT):
        self.end_time += time

==> pycaptions-0.5.0-py3-none-any.whl: pycaptions/block.py <==
import budoux
from collections import defaultdict
from langcodes import standardize_tag, tag_is_valid
from .microTime import MicroTime as MT
from .styling import Styling


class BlockType:
    CAPTION = 1
    COMMENT = 2
    STYLE = 3
    LAYOUT = 4
    METADATA = 5


class Block:
    """
    Represents a block of content in a multimedia file or document.

    Methods:
        getLines: Format text of specific language into multiple lines
        get: Get the text content of the block for a specific language.
        append: Append text to the block for a specific language.
        shift_time: Shift start and end times of the block by a specified duration.
        shift_start: Shift start time of the block by a specified duration.
        shif_end: Shift end time of the block by a specified duration.
        copy: Returns a copy of the current block.
        shift_time_us: Shift time of the block by microseconds.
        shift_start_us: Shift start time of the block by microseconds.
        shift_end_us: Shift end time of the block by microseconds.
        __getitem__: Retrieve the text of specific language.
        __setitem__: Set the text of specific language.
        __str__: Return a string representation of the block.
        __iadd__: In-place addition for the Block.
        __add__: Addition for the Blocks.
        __isub__: In-place subtraction for a specific language.
        __sub__: Subtraction for a specific language.
        __iter__: Iterator for iterating through the block languages.
        __next__: Iterator method returning a tuple of language and text.
    """
    def __init__(self, block_type: int, lang: str = "und", start_time: MT = None,
                 end_time: MT = None, text: str = "", **options):
        """
        Initialize a new instance of the Block class.

        Parameters:
        - block_type (int): The type of the block, represented as an integer, options in BlockType class
        - lang (str, optional): The language of the text in the block (default is "und" for undefined).
        - start_time (int, optional): The starting time of the block in microseconds (default is 0).
        - end_time (int, optional): The ending time of the block in microseconds (default is 0).
        - text (str, optional): The content of the block (default is an empty string).
        - **options: Additional keyword arguments for customization (e.g style, layout, ...).
        """
        self.block_type = block_type
        self.languages = defaultdict(str)
        if options.get("languages"):
            for i, j in options.get("languages").items():
                self.languages[i] = j
            del options["languages"]
        self.default_language = lang
        if text:
            self.languages[lang] = text.strip()
        self.start_time = start_time
        self.end_time = end_time
        self.options = options or {}

    def __getitem__(self, index: str):
        return self.languages[index]

    def __setitem__(self, index: str, value: str):
        self.languages[index] = value

    def __delitem__(self, index: str):
        del self.languages[index]

    def __str__(self):
        temp = '\n'.join(f" {lang}: {text}" for lang, text in self.languages.items())
        return f"start: {self.start_time} end: {self.end_time}\n{temp}"

    def __iadd__(self, value):
        if not isinstance(value, Block):
            raise ValueError("Unsupported type. Must be an instance of `Block`")
        for key, language in value:
            self.languages[key] = language
        return self

    def __add__(self, value):
        if not isinstance(value, Block):
            raise ValueError("Unsupported type. Must be an instance of `Block`")
        out = Block(block_type=self.block_type, start_time=self.start_time,
                    end_time=self.end_time, lang=self.default_language, options=self.options)
        out.languages = self.languages.copy()
        for key, language, comment in value:
            out.languages[key] = language
        return out

    def __isub__(self, language: str):
        if language in self.languages:
            del self.languages[language]
        return self

    def __sub__(self, language: str):
        out = Block(block_type=self.block_type, start_time=self.start_time,
                    end_time=self.end_time, lang=self.default_language, options=self.options)
        out.languages = self.languages.copy()
        if language in out.languages:
            del out.languages[language]
        return out

    def __iter__(self):
        self._keys_iterator = iter(self.languages)
        return self

    def __next__(self):
        try:
            key = next(self._keys_iterator)
            return key, self.languages.get(key)
        except StopIteration:
            raise StopIteration

    def copy(self):
        return Block(self.block_type, self.default_language, self.start_time,
                     self.end_time, languages=self.languages, **self.options)   

    def get(self, lang: str) -> str:
        return Styling(self.languages.get(lang), "html.parser").get_text()
    
    def get_style(self, lang: str) -> str:
        return Styling(self.languages.get(lang), "html.parser")
    
    def getLines(self, lang:str = None, lines: int = 0, character_limit: int = 47, split_ratios: list[float] = [0.7, 1], **kwargs) -> list[str]:
        """
        Format text of specific language into multiple lines.

        Args:
            lang (str, optional): Language code (default is None - uses default_language).
            lines (int, optional): The number of lines to format to. (default is 0 - autoformat). Ignores character limit and split ratios if it cannot fit in the desired amount.
            character_limit (int, optional) How many characters should be in a line. (default is 47)
            split_ratios (list[float], optional): Affects character_limit for n-th line. (default [0.7, 1])
            parser_language (str, optional): Parser language code, if None it uses lang.
        Returns:
            list[str]: A list of text lines.
        """
        lang = lang or self.default_language
        text = self.get(lang)

        standardized = standardize_tag(kwargs.get("parser_language") or lang, macro=True)
        standardized = standardized if tag_is_valid(standardized) else "und"
        
        if lines == 1:
            return [text]

        if standardized == "ja":
            parser = budoux.load_default_japanese_parser()
            phrases = parser.parse(text)
        elif standardized in ["zh", "zh-CN", "zh-SG", "zh-Hans"]:
            parser = budoux.load_default_simplified_chinese_parser()
            phrases = parser.parse(text)
        elif standardized in ["zh-HK", "zh-MO", "zh-TW", "zh-Hant"]:
            parser = budoux.load_default_simplified_chinese_parser()
            phrases = parser.parse(text)
        elif standardized == "th":
            parser = budoux.load_default_thai_parser()
            phrases = parser.parse(text)
        else:
            phrases = text.split(" ")

        if lines != 0:
            total_characters = len(text)
            target_characters = total_characters - lines + 1
            current_limit = sum(character_limit * ratio for ratio in split_ratios)
            if current_limit < target_characters:
                remaining = (target_characters - current_limit) / total_characters
                for index, _ in enumerate(split_ratios):
                    split_ratios[index] += remaining  

        formatted_lines = []
        current_line = ""
        current_character_count = 0

        for phrase in phrases:
            current_ratio_index = min(len(formatted_lines), len(split_ratios) - 1)
            effective_limit = int(character_limit * split_ratios[current_ratio_index])

            if current_character_count + len(phrase) <= effective_limit:
                current_line += phrase + " "
                current_character_count += len(phrase) + 1  # +1 for the space
            else:
                formatted_lines.append(current_line.strip())
                current_line = phrase + " "
                current_character_count = len(phrase) + 1

        if current_line:
            formatted_lines.append(current_line.strip())

        return formatted_lines

    def append(self, text: str, lang: str = None):
        lang = lang or self.default_language
        if lang not in ["ja", "zh", "zh-CN", "zh-SG", "zh-Hans",
                        "zh-HK", "zh-MO", "zh-TW", "zh-Hant"]:
            if self.languages[lang]:
                self.languages[lang] += " " + text.strip()
            else:
                self.languages[lang] = text.strip()
        else:
            self.languages[lang] += text

    def append_without_common_part(self, text: str, lang: str = None):
        lang = lang or self.default_language
        common_lenght = 0
        current = self.get(lang)
        min_length = min(len(current), len(text))

        for i in range(min_length):
            if current[-i:] == text[:i]:
                common_lenght = i
        
        self.languages[lang] = current + text[common_lenght:]

    def shift_time_us(self, microseconds: int):
        self.start_time += microseconds
        self.end_time += microseconds

    def shift_time(self, time: MT):
        self.start_time += time
        self.end_time += time

    def shift_start_us(self, microseconds: int):
        self.start_time += microseconds

    def shift_start(self, time: MT):
        self.start_time += time

    def shift_end_us(self, microseconds: int):
        self.end_time += microseconds

    def shift_end(self, time: MT):
        self.end_time += time
